@@ -1,10 +1,10 @@
 # -*- perl -*-
 
 #
-# $Id: Dialog.pm,v 1.9 2004/09/04 01:11:59 eserte Exp $
+# $Id: Dialog.pm,v 1.10 2007/08/29 16:18:50 k_wittrock Exp $
 # Author: Slaven Rezic
 #
-# Copyright (C) 2001 Slaven Rezic. All rights reserved.
+# Copyright (C) 2001,2005 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -17,7 +17,7 @@ use Tk::PathEntry;
 use base qw(Tk::DialogBox);
 use strict;
 use vars qw($VERSION);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.9 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.10 $ =~ /(\d+)\.(\d+)/);
 
 Construct Tk::Widget 'PathEntryDialog';
 
@@ -43,20 +43,16 @@ sub Populate {
     my($w, $args) = @_;
 
     $args->{-buttons} = ["OK", "Cancel"];
-
-    my %pe_args;
-    foreach (qw(dir file)) {
-	if (exists $args->{"-initial$_"}) {
-	    $pe_args{"-initial$_"} = delete $args->{"-initial$_"};
-	}
-    }
-
+    # Disable default button feature of Tk::DialogBox
+    $args->{-default_button} = 'none';
+    $args->{-title}  ||=  'Select path';   # default window title
     $w->SUPER::Populate($args);
 
     my $pe = $w->add('PathEntry',
 		     -textvariable => \$w->{PathName},
-		     %pe_args)->pack;
+		    )->pack(-expand => 1, -fill => 'x');
     $w->Advertise("PathEntry" => $pe);
+    $args->{-focus} = $pe;
 
     $pe->bind("<Return>" => sub {
 		  $w->Subwidget("B_OK")->Invoke;
@@ -66,7 +62,9 @@ sub Populate {
 	     });
 
     $w->ConfigSpecs
-	(-create => ['PASSIVE', undef, undef, 0]);
+	(-create => ['PASSIVE', undef, undef, 0],
+	 'DEFAULT' => [$pe],
+	);
 }
 
 sub Show {
@@ -79,20 +77,20 @@ sub Show {
     while (1) {
 	undef $pathname;
 
-	$w->after(300, sub {
-		      $pe->focus;
-		      $pe->icursor("end");
-		  });
-
 	my $r = $w->SUPER::Show(@args);
 	$pathname = $w->{PathName} if $r =~ /ok/i;
 	$pe->Finish;
 
-	if (defined $pathname && $w->cget(-create) && -e $pathname) {
+	if (defined $pathname && $w->cget(-create) && -f $pathname) {
 
+	    # Disable default button feature of Tk::DialogBox
+	    # (invalid option and not required for Windows)
+	    my $noDefault = $Tk::platform eq 'MSWin32' ? '' :
+                               "-default_button => 'none'";
 	    my $reply = $w->messageBox
 		(-icon => 'warning',
 		 -type => 'YesNo',
+		 eval {split(' => ', $noDefault)},
 		 -message => "File \"$pathname\" already exists.\nDo you want to overwrite it?");
 	    redo unless (lc($reply) eq 'yes');
 	}
@@ -125,17 +123,67 @@ Tk::PathEntry::Dialog - File dialog using Tk::PathEntry
 
 =head1 SYNOPSIS
 
+Using as a replacement for getOpenFile and getSaveFile:
+
     use Tk::PathEntry::Dialog qw(as_default);
-    $mw->getOpenFile;
+    $filename = $mw->getOpenFile;
+
+Using as a normal module:
+
+    use Tk::PathEntry::Dialog;
+    $filename = $mw->PathEntryDialog(-autocomplete => 1)->Show;
 
 =head1 DESCRIPTION
 
+This module provides a dialog window with a L<Tk::PathEntry|Tk::PathEntry>
+widget, an OK button and a Cancel button.
+
 With this module, the L<Tk::PathEntry|Tk::PathEntry> can also be used
-as a standard Tk file dialog.
+as a standard Tk file dialog. You are allowed to select a directory.
+
+=head1 OPTIONS
+
+=head2 Options of getOpenFile and getSaveFile
+
+You cannot use the options C<-filetypes>, C<-defaultextension>, and C<-multiple>. So the only
+remaining options are C<-initialdir>, C<-initialfile>, and C<-title>.
+
+=head2 Options of PathEntryDialog
+
+B<PathEntryDialog> supports all options of L<Tk::PathEntry|Tk::PathEntry>. The additional options are:
+
+=over 4
+
+=item -title
+
+Sets the window title.
+
+=item -create
+
+If this is set to a C<true> value, you will be warned when you select an existing file.
+
+=back
+
+=head1 NOTES
+
+Surprisingly this module also works on Microsoft Windows.
 
 =head1 BUGS
 
-This widget does not work on Windows.
+The following bug is known for B<PathEntryDialog> on Microsoft Windows:
+Directly after klicking on a choice in the choices listbox which displays
+below the C<PathEntryDialog> window, the C<OK> and
+C<Cancel> buttons don't respond to mouse clicks. Workaround: Move the mouse
+cursor out of the button and back. Or use the Enter resp. Escape keys in
+place of the buttons.
+
+The following bug is known for B<PathEntryDialog> on Knoppix:
+C<PathEntryDialog> will often abort immediately with the error message
+
+	*** glibc detected *** malloc(): memory corruption: 0x08495514 ***
+
+Workaround: Do something that causes B<PathEntryDialog> to start at a different 
+memory location, e. g. open or close a Konqueror window (beleave me, this may help).
 
 =head1 SEE ALSO
 
